from enum import Enum


class ReadRecordRequestType(Enum):
    """
    Enum class to ensure accuracy of read record request types
    """
    SAMPLE = 'SAMPLE'
    UNREAD = 'UNREAD'
    READ_SINCE_RECORD_ID = 'READ_SINCE_RECORD_ID'
    READ_SINCE_TIMESTAMP = 'READ_SINCE_TIMESTAMP'
    SINGLE = 'SINGLE'
    LAST_N = 'LAST_N'


from streamduo.models.batch_data import BatchData
from streamduo.models.batch_init_request import BatchInitRequest
import hashlib
import os
import base64

from streamduo.models.schema import SchemaType

from streamduo.models.schema import FileType
from streamduo.validators.json_schema import JsonValidator


class BatchController:
    """
    Provides methods for interacting with the `/batch` endpoints
    """
    BUFFER_SIZE = 1024 * 1024 * 5  # 5 MB

    def __init__(self, client):
        self.client = client

    @staticmethod
    def construct_batch_init_request(file_path):
        """
        constructs the payload for the initiate batch file upload request
        :param file_path: (STRING) path to file for upload.
        :return: (BatchInitRequest) object.
        """
        # construct request
        batch_init_request = BatchInitRequest()
        part_number = 1
        # break down chunks
        with open(file_path, 'rb') as out_file:
            full_file_md5 = hashlib.md5()
            while True:
                data = out_file.read(BatchController.BUFFER_SIZE)
                if not data:
                    break
                # update full file Hash
                full_file_md5.update(data)
                # add part hash as b64 to batch init req
                batch_init_request.hashes[part_number] = base64.b64encode(
                    hashlib.md5(data).digest()).decode()
                part_number = part_number + 1

        batch_init_request.hashValue = base64.b64encode(full_file_md5.digest()).decode()
        batch_init_request.fileName = os.path.basename(file_path)
        filename, file_extension = os.path.splitext(file_path)
        file_extension = file_extension.strip('.').upper()
        if file_extension in FileType._member_names_:
            batch_init_request.fileType = FileType[file_extension].value
        batch_init_request.totalParts = part_number
        return batch_init_request

    @staticmethod
    def get_part_binary(file_path, part_number):
        """
        returns the bytes for a particular "part number" of a file.
        :param file_path: (STRING) path to file for upload.
        :param part_number: (INT) part number to return
        :return: (BYTES) bytes of data corresponding to the file part.
        """
        part_counter = 1
        with open(file_path, 'rb') as out_file:
            while part_counter <= int(part_number):
                data = out_file.read(BatchController.BUFFER_SIZE)
                # EOF
                if not data:
                    break
                part_counter = part_counter + 1
            return data

    def send_batch_init(self, stream_id, file_path):
        """
        Sends a Batch initiation request to the API.
        :param stream_id: (STRING) Stream ID file will be uploaded to.
        :param file_path: (STRING) Path to file being uploaded.
        :return:
        """
        request_object = BatchController.construct_batch_init_request(file_path=file_path)
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/batch/init",
                                    body=request_object.to_json())

    def send_batch_part(self, batch_data, part_number, binary_payload):
        file_payload = {'file': binary_payload}
        return self.client.call_api('POST',
                                    f"/stream/{batch_data.streamId}/batch/{batch_data.batchId}/upload-part/{part_number}",
                                    files=file_payload)

    def send_file(self, stream_id, file_path):
        ## init req
        batch_data = BatchData(**self.send_batch_init(stream_id=stream_id,
                                                      file_path=file_path).json())

        ## CSV w. JSON Schema
        if batch_data.requires_validation() \
                and batch_data.get_file_type() == FileType.CSV \
                and batch_data.get_stream_schema().get_schema_type() == SchemaType.JSON:
            ## validate
            val = JsonValidator()
            val.set_schema(batch_data.get_stream_schema().get_schema())
            val.validate_csv(file_path)

        # send via loop chunks
        with open(file_path, 'rb') as out_file:
            while len(batch_data.outstandingParts.keys()) > 0:
                part_number = next(iter(batch_data.outstandingParts.items()))[0]
                data = BatchController.get_part_binary(file_path=file_path, part_number=part_number)
                # send data
                print(f"""sending part: {part_number}""")
                batch_data = BatchData(**self.send_batch_part(batch_data=batch_data,
                                                              part_number=part_number, binary_payload=data).json())
        return batch_data



from unittest import TestCase

import jsonschema

from streamduo.validators.json_schema import JsonValidator


class TestJson(TestCase):

    def test_json_record_validation(self):
        val = JsonValidator()
        val.set_local_schema("../test_schemas/car_schema.json")
        valid_rec = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": "31600",
            "Colour": "White",
            "Doors": "4"
        }
        assert val.validate_record(valid_rec)
        inval_rec = valid_rec.copy()
        inval_rec['extra'] = "xyz"
        with self.assertRaises(jsonschema.exceptions.ValidationError):
            val.validate_record(inval_rec)

    def test_json_list_validation(self):
        val = JsonValidator()
        val.set_local_schema("../test_schemas/car_schema.json")
        valid_rec_1 = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": "31600",
            "Colour": "White",
            "Doors": "4"
        }
        valid_rec_2 = {
            "Make": "Honda",
            "Price": "$9,700.00",
            "Odometer (KM)": "31600",
            "Colour": "White",
            "Doors": "4"
        }
        rec_list = [valid_rec_1, valid_rec_2]
        assert val.validate_list(rec_list)

        ## Make Bad
        rec_list[1]['extra'] = "xyz"
        with self.assertRaises(jsonschema.exceptions.ValidationError):
            val.validate_list(rec_list)

    def test_bad_schema(self):
        val = JsonValidator()
        with self.assertRaises(jsonschema.exceptions.SchemaError):
            val.set_schema("asdf")

    def test_csv_validation(self):
        val = JsonValidator()
        val.set_local_schema("../test_schemas/car_schema.json")
        assert val.validate_csv("../test_data/car_sales.csv")
        with self.assertRaises(jsonschema.exceptions.ValidationError):
            val.validate_csv("../test_data/car_sales_bad.csv")



import os
from unittest import TestCase
from streamduo.validators.frictionless_schema import validate_schema, FrictionValidator
from frictionless import describe
import pprint as pp


class TestFriction(TestCase):

    def test_csv_validation(self):
        val = FrictionValidator()
        val.set_schema("../test_schemas/car_schema.friction")
        val.validate_csv("unit_test_data/car_sales.csv")




import json
import csv
from jsonschema import validate, validators


def validate_schema(schema):
    return validators.Draft7Validator.check_schema(schema)


class JsonValidator:
    schema = None
    int_fields = []
    float_fields = []
    """
    Provides methods for validating records and files against a given AVRO schema
    """

    def __init__(self):
        pass

    def set_local_schema(self, schema_file):
        try:
            with open(schema_file, 'r') as file:
                self.set_schema(json.load(file))
        except SyntaxError as error:
            raise

    def set_schema(self, schema):
        self.schema = schema
        print(validators.Draft7Validator.check_schema(self.schema))
        # integer :: int
        # number :: float
        for k, v in self.schema['properties'].items():
            if v['type'] == 'integer':
                self.int_fields.append(k)
            elif v['type'] == 'number':
                self.float_fields.append(k)

    def validate_record(self, record):
        ## retype numerics
        for x in self.int_fields:
            record[x] = int(record[x])
        for y in self.float_fields:
            record[y] = float(record[y])
        return validate(record, self.schema) is None

    def validate_list(self, record_list):
        for r in record_list:
            if not self.validate_record(r):
                return False
        return True

    def validate_csv(self, csv_file_path):
        with open(csv_file_path) as csv_file:
            csv_reader = csv.DictReader(csv_file)
            for r in csv_reader:
                if not self.validate_record(r):
                    return False
        return True



from time import sleep
from unittest import TestCase
import os
from streamduo.client import Client, PublicClient
import pandas as pd
import json

class TestRecord(TestCase):
    def test_write_record(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        payload = {'level_1a': [{'level_2a': 99},
                               {'level_2b': 'wayne'}],
                   'level_1b': 'some text'
                   }
        write_response = record_controller.write_record(stream_id, payload)
        assert write_response.json()['dataPayload']['level_1b'] == 'some text'

        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_write_record_pandas(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()

        car_sales = pd.read_csv("car_sales.csv")
        for index, row in car_sales.iterrows():
            write_response = record_controller.write_record(stream_id, row.to_json())

        read_records_response = record_controller.read_unread_records(stream_id=stream_id, mark_as_read=False,
                                                                      record_count=100)
        assert len(read_records_response.json()) == 10

        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_write_record_pandas_blob(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()

        car_sales = pd.read_csv("car_sales.csv")
        car_sales_json = json.loads(car_sales.loc[1:3].to_json(orient="index"))
        write_response = record_controller.write_record(stream_id, car_sales_json)

        read_records_response = record_controller.read_unread_records(stream_id=stream_id, mark_as_read=False,
                                                                      record_count=100)
        assert len(read_records_response.json()[0]['dataPayload']) == 3

        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_write_csv_records(self):
        file = 'car_sales.csv'
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        with open(file, 'rb') as file_stream:
            write_file_response = record_controller.write_csv_records(stream_id=stream_id, file_stream=file_stream)
        assert write_file_response.json()['recordsWritten'] == 10

        read_records_response = record_controller.read_unread_records(stream_id=stream_id, mark_as_read=False, record_count=100)
        assert len(read_records_response.json()) == 10
        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_read_simple_record(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        payload = {'level_1a': [{'level_2a': 99},
                                {'level_2b': 'wayne'}],
                   'level_1b': 'some text'
                   }
        write_response = record_controller.write_record(stream_id, payload)
        record_id = write_response.json()['recordId']

        read_response = record_controller.read_record(stream_id, record_id, False)
        assert len(read_response.json()) == 1
        assert read_response.json()[0]['dataPayload']['level_1b'] == 'some text'

        ## Cleanup
        stream_controller.delete_stream(stream_id)


    def test_unread_records(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        payload_1 = {'payload': 1}

        payload_2 = {'payload': 2}

        payload_3 = {'payload': 3}

        record_id_1 = record_controller.write_record(stream_id, payload_1).json()['recordId']
        record_id_2 = record_controller.write_record(stream_id, payload_2).json()['recordId']
        record_id_3 = record_controller.write_record(stream_id, payload_3).json()['recordId']

        read_unread_response = record_controller.read_unread_records(stream_id, False, 5)
        assert len(read_unread_response.json()) == 3

        read_one_response = record_controller.read_record(stream_id,record_id_2, True)

        read_unread_response = record_controller.read_unread_records(stream_id, True, 5)
        assert len(read_unread_response.json()) == 2

        read_unread_response = record_controller.read_unread_records(stream_id, False, 5)
        assert len(read_unread_response.json()) == 0

        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_simple_unread_records(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        payload_1 = {'payload': 1}

        payload_2 = {'payload': 2}

        payload_3 = {'payload': 3}

        record_id_1 = record_controller.write_record(stream_id, payload_1).json()['recordId']
        record_id_2 = record_controller.write_record(stream_id, payload_2).json()['recordId']
        record_id_3 = record_controller.write_record(stream_id, payload_3).json()['recordId']

        read_unread_response = record_controller.read_unread_records(stream_id, False, 5)
        assert len(read_unread_response.json()) == 3

        read_one_response = record_controller.read_record(stream_id, record_id_2, True)

        read_unread_response = record_controller.simple_read_unread_records(stream_id)
        assert len(read_unread_response.json()) == 2

        #Try again, should be all read and no results
        read_unread_response = record_controller.simple_read_unread_records(stream_id)
        assert len(read_unread_response.json()) == 0

        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_last_n_records(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        payload_1 = {'payload': 1}

        payload_2 = {'payload': 2}

        payload_3 = {'payload': 3}

        record_id_1 = record_controller.write_record(stream_id, payload_1).json()['recordId']
        record_id_2 = record_controller.write_record(stream_id, payload_2).json()['recordId']
        record_id_3 = record_controller.write_record(stream_id, payload_3).json()['recordId']

        read_unread_response = record_controller.read_last_n_records(stream_id, False, 2)
        assert len(read_unread_response.json()) == 2
        record_list = []
        for r in read_unread_response.json():
            record_list.append(r['recordId'])
        assert record_id_3 in record_list
        assert record_id_2 in record_list
        assert record_id_1 not in record_list

        read_unread_response = record_controller.read_last_n_records(stream_id, False, 10)
        assert len(read_unread_response.json()) == 3
        record_list = []
        for r in read_unread_response.json():
            record_list.append(r['recordId'])
        assert record_id_3 in record_list
        assert record_id_2 in record_list
        assert record_id_1 in record_list

        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_read_since_timestamp(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        payload_1 = {'payload': 1}
        payload_2 = {'payload': 2}
        payload_3 = {'payload': 3}

        record_1 = record_controller.write_record(stream_id, payload_1).json()
        sleep(3)
        record_2 = record_controller.write_record(stream_id, payload_2).json()
        sleep(3)
        record_3 = record_controller.write_record(stream_id, payload_3).json()

        read_timestamp_response = record_controller.read_records_since_timestamp(stream_id, record_2['recordTimeStampISO'], False, 5)
        assert len(read_timestamp_response.json()) == 1
        record_list = []
        for r in read_timestamp_response.json():
            record_list.append(r['recordId'])
        assert record_1['recordId'] not in record_list
        assert record_2['recordId'] not in record_list
        assert record_3['recordId'] in record_list

        #Trim timestamp to the second
        read_timestamp_response = record_controller.read_records_since_timestamp(stream_id, record_2['recordTimeStampISO'][:19], False, 5)
        assert len(read_timestamp_response.json()) == 2
        record_list = []
        for r in read_timestamp_response.json():
            record_list.append(r['recordId'])
        assert record_1['recordId'] not in record_list
        assert record_2['recordId'] in record_list
        assert record_3['recordId'] in record_list

        ## Cleanup
        stream_controller.delete_stream(stream_id)


    def test_add_to_record_id(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']

        record_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()
        record_id = '111111111'

        payload = {
            'Part Description': 'Widget A',
            'Price': 10.00,
            'Inventory': 20000
        }

        write_response = record_controller.write_record(stream_id, payload, record_id=record_id)

        #write again to same ID
        payload = {
            'Part Description': 'Widget A',
            'Price': 11.00,
            'Inventory': 15000
        }
        write_response2 = record_controller.write_record(stream_id, payload, record_id=record_id)

        assert write_response2.json()['recordId'] == record_id

        ##get a record ID returns 2
        read_response = record_controller.read_record(stream_id, record_id, False)
        assert len(read_response.json()) == 1
        assert read_response.json()[0]['dataPayload']['Inventory'] == 15000

        read_hist_response = record_controller.read_record_hist(stream_id=stream_id, record_id=record_id, count=3, mark_as_read=True)
        assert len(read_hist_response.json()) == 2


        ## Cleanup
        stream_controller.delete_stream(stream_id)

    def test_write_record_errors(self):
        record_controller = Client(os.getenv('AUTH_CLIENT_ID'),
                                   os.getenv('AUTH_CLIENT_SECRET')).get_record_controller()

        #stream does not exist
        payload = {'level_1a': [{'level_2a': 99},
                                {'level_2b': 'wayne'}],
                   'level_1b': 'some text'
                   }
        write_response = record_controller.write_record("madeupsstreamid", payload)
        assert write_response.status_code == 401

    def test_write_record_public(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']
        ## Make public
        client_display_name = 'test_client'
        add_client_response = stream_controller.add_public_client_to_stream(stream_id=stream_id, client_display_name=client_display_name, is_producer=True)
        ##get new client ID
        new_client_id = None
        for i in add_client_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == client_display_name:
                new_client_id = i['actorId']
                break
        assert new_client_id is not None

        ## public write
        record_controller = PublicClient().get_record_controller()
        payload = {'level_1a': [{'level_2a': 99},
                               {'level_2b': 'wayne'}],
                   'level_1b': 'some text'
                   }
        write_response = record_controller.write_record(stream_id, payload)
        assert write_response.json()['dataPayload']['level_1b'] == 'some text'

        ## Remove Public Client ID from stream permissions
        remove_client_response = stream_controller.remove_machine_client_from_stream(stream_id, new_client_id)
        check_client_id = None
        for i in remove_client_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == client_display_name:
                check_client_id = i['actorId']
                break
        assert check_client_id is None
        assert len(remove_client_response.json()['streamActorPermissionRecords']) == 1

        ## Cleanup
        stream_controller.delete_stream(stream_id)







from enum import Enum


class KeyAlgorithm(Enum):
    CURVE25519 = 'CURVE25519'

class KeyEncoding(Enum):
    BASE64 = 'BASE64'


class KeyPair:
    """
    Class for keypairs
    """
    def __init__(self, publicKeyId=None, publicKeyAlgorithm=None, publicKeyVersion=None, publicKeyEncoding=None, publicKey=None, publicKeyDescription=None ):
        """
         Constructor
         """
        self.publicKeyId = publicKeyId
        self.publicKeyAlgorithm = publicKeyAlgorithm
        self.publicKeyVersion = publicKeyVersion
        self.publicKeyEncoding = publicKeyEncoding
        self.publicKey = publicKey
        self.publicKeyDescription = publicKeyDescription

    def to_json(self):
        return self.__dict__


import json
from enum import Enum


class SchemaType(Enum):
    JSON = 'JSON'
    AVRO = 'AVRO'
    GREAT_EXPECTATIONS = 'GREAT_EXPECTATIONS'


class FileType(Enum):
    JSON = 'JSON'
    AVRO = 'AVRO'
    CSV = 'CSV'


class Schema:
    """
    Class used to aid in the use of schema objects
    """

    def __init__(self, **entries):
        self.__dict__.update(entries)

    def to_json(self):
        return self.__dict__

    def get_schema(self):
        return json.loads(self.schema)

    def get_schema_type(self):
        return SchemaType[self.schemaType]




from unittest import TestCase

from nacl.public import SealedBox

from streamduo.api.key import KeyController


class TestKey(TestCase):

    def test_create_key(self):
        pub, priv = KeyController.create_key_local()
        assert len(priv) == 44

    def test_encr_decr(self):
        ##Key Object, priv key (string)
        pub, priv = KeyController.create_key_local()
        sb = SealedBox(KeyController.get_public_key(pub.publicKey))
        with open("../test_data/car_sales.csv", 'rb') as file:
            encrypted_data = sb.encrypt(file.read())
        unseal_box = SealedBox(KeyController.get_private_key(priv))
        out = unseal_box.decrypt(encrypted_data)
        print(out.decode('utf-8'))






import json

from streamduo.models.read_record_request import ReadRecordRequest
from streamduo.models.record import Record
from streamduo.models.record_request import ReadRecordRequestType


class RecordController:
    """
    This class manages the reading and writing of records to streams.
    """

    def __init__(self, client):
        self.client = client

    def write_record(self, stream_id, json_payload, record_id=None):
        """
        Writes a single record to the stream.
        :param stream_id: (String) Stream ID to write to
        :param json_payload: (Dict OR JSON formatted String) The payload of the record
        :param record_id: (String) OPTIONAL - Used in the case you want to force a particular record ID.
        If left null, record ID will be generated by API
        :return: (Requests Response) API response, body will be a Record object Dict.
        """
        record = Record()
        record.recordId = record_id
        if isinstance(json_payload, str):
            record.dataPayload = json.loads(json_payload)
        else:
            record.dataPayload = json_payload
        return self.client.call_api("POST",
                                    f"/stream/{stream_id}/record",
                                    body=record.to_json()
                                    )

    def write_csv_records(self, stream_id, file_stream):
        """
        Writes a CSV file of records to a stream.
        :param stream_id: (String) Stream ID to write to
        :param file_stream: (File) File Object of CSV
        :return:  (Requests Response) API response, body will be a String detailing the status.
        """
        file_payload = {'file': file_stream}
        return self.client.call_api("POST",
                                    f"/stream/{stream_id}/record/batch-file",
                                    files=file_payload
                                    )

    def read_record(self, stream_id, record_id, mark_as_read):
        """
        Reads the latest version of a specific record in a stream by Record ID
        :param stream_id: (String) Stream to read from
        :param record_id: (String) Record ID to read
        :param mark_as_read: (Bool) Whether the record should be marked as read
        :return:  (Requests Response) API response, body will be LIST of Record object Dicts.
        """
        read_record_request = ReadRecordRequest()
        read_record_request.recordId = record_id
        read_record_request.readRecordRequestType = ReadRecordRequestType.SINGLE
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = 1
        return self._read_record_request(stream_id, read_record_request)

    def read_record_hist(self, stream_id: str, record_id: str, count: int, mark_as_read: bool):
        """
        Reads historical records from a stream, for a given record ID.
        :param stream_id: (String) Stream to read from
        :param record_id: (String) Record ID to read
        :param count: (Int) Number of historical records to read for (going back in time)
        :param mark_as_read: (Bool) Whether the record should be marked as read
        :return:  (Requests Response) API response, body will be LIST of Record object Dicts.
        """

        read_record_request = ReadRecordRequest()
        read_record_request.recordId = record_id
        read_record_request.readRecordRequestType = ReadRecordRequestType.SINGLE
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = count
        return self._read_record_request(stream_id, read_record_request)

    def read_unread_records(self, stream_id, mark_as_read, record_count):
        """
        Reads unread records from a stream, returning records in chronological order
        from the last read record.
        :param stream_id: (String) Stream to read from
        :param mark_as_read: (Bool) Whether the record should be marked as read
        :param record_count: (Int) Number of records to read
        :return: (Requests Response) API response, body will be LIST of Record object Dicts.
        """
        read_record_request = ReadRecordRequest()
        read_record_request.readRecordRequestType = ReadRecordRequestType.UNREAD
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = record_count
        return self._read_record_request(stream_id, read_record_request)

    def simple_read_unread_records(self, stream_id, record_count=None):
        """
        Reads unread records from a stream, returning records in chronological order
        from the last read record. Always marks as read.
        :param stream_id: (String) Stream to read from
        :param record_count: (Int) Optional: Number of records to read, will use server default when omitted.
        :return: (Requests Response) API response, body will be LIST of Record object Dicts.
        """
        params = {}
        if record_count:
            params["count"] = record_count

        return self.client.call_api("GET",
                                    f"/stream/{stream_id}/record/unread",
                                    params=params
                                    )

    def read_last_n_records(self, stream_id, mark_as_read, record_count):
        """
        Reads the last {record_count} records in a stream
        :param stream_id: (String) Stream to read from
        :param mark_as_read:  (Bool) Whether the record should be marked as read
        :param record_count: (Int) Number of records to read
        :return: (Requests Response) API response, body will be LIST of Record object Dicts.
        """
        read_record_request = ReadRecordRequest()
        read_record_request.readRecordRequestType = ReadRecordRequestType.LAST_N
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = record_count
        return self._read_record_request(stream_id, read_record_request)

    def read_records_since_timestamp(self, stream_id, timestamp, mark_as_read, record_count):
        """
        Reads records in stream since a given timestamp
        :param stream_id: (String) Stream to read from
        :param timestamp: (String) Timestamp to start reading from (ISO format) i.e. 2021-11-03T21:03
        :param mark_as_read: (Bool) Whether the record should be marked as read
        :param record_count: (Int) Number of records to read
        :return: (Requests Response) API response, body will be LIST of Record object Dicts.
        """
        read_record_request = ReadRecordRequest()
        read_record_request.readRecordRequestType = ReadRecordRequestType.READ_SINCE_TIMESTAMP
        read_record_request.recordTimeStampISO = timestamp
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = record_count
        return self._read_record_request(stream_id, read_record_request)

    def _read_record_request(self, stream_id, read_record_request):
        """
        Internal method for executing the read records request
        :param stream_id: (String) Stream Id to read from
        :param read_record_request: (ReadRecordRequest) object representing the read records request
        :return: (Requests Response) API response, body will be LIST of Record object Dicts.
        """
        read_record_request.readRecordRequestType = read_record_request.readRecordRequestType.value
        return self.client.call_api("POST",
                                    f"/stream/{stream_id}/record-request",
                                    body=read_record_request.to_json
                                    )





import responses
from unittest import TestCase
from responses.matchers import json_params_matcher
from streamduo import Client


class TestActor(TestCase):

    @responses.activate
    def test_get_machine_client(self):
        responses.add(responses.POST, Client.auth_endpoint,
                      json={'access_token': "fake-token"}, status=200)
        client = Client("client_id", "client_secret")
        client_id = "cid_001"
        resp_payload = {"clientId": client_id,
                            "clientDisplayName": "client display name"}

        responses.add(responses.GET, f"{client.api_endpoint}/client/{client_id}",
                      json = resp_payload,
                      status=200)
        actor_controller = client.get_actor_controller()
        assert actor_controller.get_machine_client(client_id).json() == resp_payload

    @responses.activate
    def test_create_machine_client(self):
        responses.add(responses.POST, Client.auth_endpoint,
                      json={'access_token': "fake-token"}, status=200)
        client = Client("client_id", "client_secret")
        client_display_name = "cdn"
        client_description = "cd"
        resp_payload = {"clientId": "client id",
                            "clientDisplayName": "client display name"}
        responses.add(responses.POST, f"{client.api_endpoint}/client",
                      match=[
                          json_params_matcher(
                              {
                                  'clientDisplayName': client_display_name,
                                   'clientDescription': client_description
                              }
                          )
                      ],
                      json = resp_payload,
                      status=200)
        actor_controller = client.get_actor_controller()
        assert actor_controller.create_machine_client(client_display_name, client_description).json() == resp_payload

    @responses.activate
    def test_delete_machine_client(self):
        responses.add(responses.POST, Client.auth_endpoint,
                      json={'access_token': "fake-token"}, status=200)
        client = Client("client_id", "client_secret")
        client_id = "cid_001"

        responses.add(responses.DELETE, f"{client.api_endpoint}/client/{client_id}",
                      status=200)
        actor_controller = client.get_actor_controller()
        assert actor_controller.delete_machine_client(client_id).status_code == 200


import responses
from unittest import TestCase

from streamduo.client import Client, PublicClient
from streamduo.models.read_record_request import ReadRecordRequest
from streamduo.models.record import Record
from streamduo.models.record_request import ReadRecordRequestType


class TestActor(TestCase):

    @responses.activate
    def test_write_record(self):
        responses.add(responses.POST, Client.auth_endpoint,
                      json={'access_token': "fake-token"}, status=200)
        client = Client("client_id", "client_secret")
        record_controller = client.get_record_controller()
        ## Setup
        stream_id = "streamId001"

        ## Payload
        payload = {
            'Part Description': 'Widget A',
            'Price': 10.00,
            'Inventory': 20000
        }

        ## Response
        resp_record = Record()
        resp_record.recordId = "rec_id123"
        resp_record.dataPayload = payload

        # Mock
        responses.add(responses.POST, f"{client.api_endpoint}/stream/{stream_id}/record",
                      json=resp_record.to_json(),
                      status=200)

        assert record_controller.write_record(stream_id=stream_id, json_payload=payload).status_code == 200
        assert record_controller.write_record(stream_id=stream_id, json_payload=payload).json()['dataPayload'] == payload

    @responses.activate
    def test_read_record(self):
        responses.add(responses.POST, Client.auth_endpoint,
                      json={'access_token': "fake-token"}, status=200)
        client = Client("client_id", "client_secret")
        record_controller = client.get_record_controller()
        ## Setup
        stream_id = "streamId001"
        record_id = "rec_id123"

        ## Payload
        payload = {
            'Part Description': 'Widget A',
            'Price': 10.00,
            'Inventory': 20000
        }

        ## Response
        resp_record = Record()
        resp_record.recordId = record_id
        resp_record.dataPayload = payload

        # Read Req
        read_request = ReadRecordRequest()
        read_request.readRecordRequestType = ReadRecordRequestType.SINGLE


        # Mock
        responses.add(responses.POST, f"{client.api_endpoint}/stream/{stream_id}/record-request",
                      json=[resp_record.to_json()],
                      status=200)

        assert record_controller.read_record(stream_id=stream_id, record_id=record_id, mark_as_read=True).status_code == 200
        assert record_controller.read_record(stream_id=stream_id, record_id=record_id, mark_as_read=True).json()[0]['dataPayload'] == payload

    @responses.activate
    def test_write_record_public(self):
        client = PublicClient()
        record_controller = client.get_record_controller()
        ## Setup
        stream_id = "streamId001"

        ## Payload
        payload = {
            'Part Description': 'Widget A',
            'Price': 10.00,
            'Inventory': 20000
        }

        ## Response
        resp_record = Record()
        resp_record.recordId = "rec_id123"
        resp_record.dataPayload = payload

        # Mock
        responses.add(responses.POST, f"{client.api_endpoint}/public/stream/{stream_id}/record",
                      json=resp_record.to_json(),
                      status=200)

        assert record_controller.write_record(stream_id=stream_id, json_payload=payload).status_code == 200
        assert record_controller.write_record(stream_id=stream_id, json_payload=payload).json()['dataPayload'] == payload

    @responses.activate
    def test_simple_read_unread_records(self):
        responses.add(responses.POST, Client.auth_endpoint,
                      json={'access_token': "fake-token"}, status=200)
        client = Client("client_id", "client_secret")
        record_controller = client.get_record_controller()
        ## Setup
        stream_id = "streamId001"

        ## Response
        response_list = []
        resp_record = Record()
        resp_record.recordId = "recordID1"
        resp_record.dataPayload = {
            'Part Description': 'Widget A',
            'Price': 10.00,
            'Inventory': 20000
        }
        response_list.append(resp_record)
        resp_record = Record()
        resp_record.recordId = "recordID2"
        resp_record.dataPayload = {
            'Part Description': 'Widget B',
            'Price': 110.00,
            'Inventory': 210000
        }
        response_list.append(resp_record)

        # Mock
        responses.add(responses.GET, f"{client.api_endpoint}/stream/{stream_id}/record/unread",
                      json=[x.to_json() for x in response_list],
                      status=200)

        assert record_controller.simple_read_unread_records(stream_id=stream_id).status_code == 200
        assert record_controller.simple_read_unread_records(stream_id=stream_id).json()[0][
                   'recordId'] == "recordID1"


import os
from unittest import TestCase

from streamduo.client import Client


class TestActor(TestCase):

    def setUp(self) -> None:
        self.api_client = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET'))

    def test_create_machine_client(self):

        actor_controller = self.api_client.get_actor_controller()

        create_actor_response = actor_controller.create_machine_client("client display name test",
                                                                       "client description test")
        new_client_id = create_actor_response.json()['clientId']
        assert new_client_id is not None

        ## cleanup
        delete_response = actor_controller.delete_machine_client(new_client_id)
        assert delete_response.status_code == 200

        ## check get
        get_response = actor_controller.get_machine_client(new_client_id)
        assert get_response.status_code == 404

    def test_list_machine_client(self):

        actor_controller = self.api_client.get_actor_controller()
        #clear out list
        client_list = actor_controller.get_clients()
        for client in client_list.json():
            print(client['clientId'])
            actor_controller.delete_machine_client(client['clientId'])
        client_list_new = actor_controller.get_clients()
        assert len(client_list_new.json()) == 0

        #create new client
        create_actor_response = actor_controller.create_machine_client("client display name test",
                                                                      "client description test")
        new_client_id = create_actor_response.json()['clientId']

        #get client list
        client_list = actor_controller.get_clients()

        new_client_found = False
        for client in client_list.json():
            if client['clientId'] == new_client_id:
                new_client_found = True
                break
        assert new_client_found == True

        ##cleanup
        actor_controller.delete_machine_client(new_client_id)

        client_list_new = actor_controller.get_clients()
        assert len(client_list_new.json()) == 0

    def test_delete_machine_client(self):
        # create stream
        display_name = 'test_stream'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']
        # create client on stream
        client_display_name = 'test_client'
        add_client_response = stream_controller.add_new_machine_client_to_stream(new_stream_id, client_display_name)
        ##get new client ID
        new_client_id = None
        for i in add_client_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == client_display_name:
                new_client_id = i['actorId']
                break
        # delete client
        actor_controller = self.api_client.get_actor_controller()
        actor_controller.delete_machine_client(new_client_id)
        # confirm client is null & perm not on stream anymore
        stream_response = stream_controller.get_stream(stream_id=new_stream_id)
        new_client_id = None
        for i in stream_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == client_display_name:
                new_client_id = i['actorId']
                break
        assert new_client_id is None


        # cleanup
        stream_controller.delete_stream(new_stream_id)
        assert actor_controller.get_machine_client(new_client_id).status_code == 404



from streamduo.models.schema import Schema, FileType


class BatchData:
    """
    Class for creating batch file inti request objects.
    Creating the request with a class helps ensure that the request body
    conforms to the requirements on the API side.
    """

    def __init__(self, batchId, streamId, fileName, hashes, hashValue, outstandingParts, serverSideValidation=None, totalParts=None, fileType=None, streamSchema=None):
        """
         Constructor
         """
        self.batchId = batchId
        self.streamId = streamId
        self.fileName = fileName
        self.hashes = hashes
        self.hashValue = hashValue
        self.fileType = fileType
        self.outstandingParts = outstandingParts
        self.serverSideValidation = serverSideValidation
        self.totalParts = totalParts
        self.streamSchema = streamSchema

    def to_json(self):
        return self.__dict__

    def get_stream_schema(self):
        return Schema(**self.streamSchema)

    def get_file_type(self):
        try:
            return FileType[self.fileType]
        except KeyError:
            return None

    def requires_validation(self):
        return self.streamSchema is not None



from frictionless import Schema, validate, validate_resource
import json
schema = Schema()
import pprint as pp



def validate_schema(schema):
    with open(schema, 'r') as file:
        schema_obj = json.load(file)
    s = validate_resource(schema_obj)
    pp.pprint(s)

class FrictionValidator:
    schema = None

    def __init__(self):
        pass

    def set_schema(self, schema_path):
        with open(schema_path, 'r') as file:
            self.schema = json.load(file)

    def validate_csv(self, csv_path):
        temp_schema = self.schema.copy()
        temp_schema['path'] = csv_path
        validation_results = validate_resource(temp_schema)
        return validation_results



import os
from unittest import TestCase
from streamduo.validators.great_expectations_schema import GreatExepectationsValidator
import json
import great_expectations as ge

class TestGreatExpectations(TestCase):

    def test_csv_validation(self):
        my_expectation_suite = json.load(open("/Users/stevefrensch/dev/streamduo-python-sdk/tests/test_schemas/car_expectations.json"))
        my_df = ge.read_csv(
            f"{os.getcwd()}/unit_test_data/car_sales.csv",
            expectation_suite=my_expectation_suite
        )
        validation_results = my_df.validate()
        assert validation_results['success']

        #val = GreatExepectationsValidator()
        #val.set_expectations("/Users/stevefrensch/dev/streamduo-python-sdk/tests/test_schemas/car_expectations.json")
        #val.validate_csv(f"{os.getcwd()}/unit_test_data/car_sales.csv")



class Record:
    """
    Class used to aid in the construction of record objects
    Help ensure accuracy of object when sent to API
    """

    def __init__(self):
        self.recordId = None
        self.streamId = None
        self.recordTimeStampISO = None
        self.readStatus = None
        self.dataPayload = None

    def to_json(self):
        return self.__dict__
    


import os
import uuid
import great_expectations as ge
import json
from great_expectations.core.batch import RuntimeBatchRequest
from great_expectations.profile.json_schema_profiler import JsonSchemaProfiler
from ruamel import yaml
from great_expectations.data_context.types.base import DataContextConfig, DatasourceConfig, \
    FilesystemStoreBackendDefaults
from great_expectations.data_context import BaseDataContext

class GreatExepectationsValidator:
    context = None
    expectations = None

    def __init__(self):
        data_context_config = DataContextConfig(
            store_backend_defaults=FilesystemStoreBackendDefaults(root_directory=f"{os.getcwd()}/store"),
        )
        self.context = BaseDataContext(project_config=data_context_config)

    def set_expectations(self, file_path):
        with open(file_path, 'r') as file:
            exp_suite = json.load(file)
        self.context.create_expectation_suite(overwrite_existing=True, **exp_suite)
        print(self.context.list_expectation_suites())

    def validate_csv(self, csv_path):
        run_id = str(uuid.uuid4())
        datasource_yaml = f"""
        name: {run_id}
        class_name: Datasource
        module_name: great_expectations.datasource
        execution_engine:
          module_name: great_expectations.execution_engine
          class_name: PandasExecutionEngine
        data_connectors:
            default_runtime_data_connector_name:
                class_name: RuntimeDataConnector
                batch_identifiers:
                    - default_identifier_name
            default_inferred_data_connector_name:
                class_name: InferredAssetFilesystemDataConnector
                base_directory: {csv_path}
                glob_directive: "*.csv"
                default_regex:
                  group_names:
                    - file_name
                  pattern: (.*)
        """
        self.context.test_yaml_config(datasource_yaml)
        self.context.add_datasource(**yaml.load(datasource_yaml))
        batch_request = RuntimeBatchRequest(
            datasource_name=run_id,
            data_connector_name="default_runtime_data_connector_name",
            data_asset_name=csv_path,  # This can be anything that identifies this data_asset for you
            runtime_parameters={"path": csv_path},  # Add your path here.
            batch_identifiers={"default_identifier_name": "default_identifier"},
        )
        self.context.create_expectation_suite(
            expectation_suite_name="test_suite", overwrite_existing=True
        )
        validator = self.context.get_validator(
            batch_request=batch_request, expectation_suite_name="test_suite"
        )
        print(validator.head())




import os
import pprint
from unittest import TestCase

from nacl.public import SealedBox

from streamduo.api.key import KeyController
from streamduo.client import Client
from streamduo.models.key_pair import KeyAlgorithm, KeyEncoding


class TestKeys(TestCase):

    def setUp(self) -> None:
        self.api_client = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET'))

    def test_write_key(self):
        display_name = 'test_write_key'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']

        pub_key, priv_key = KeyController.create_key_local()
        key_controller = self.api_client.get_key_controller()
        upload_response = key_controller.upload_key(stream_id=new_stream_id, key=pub_key)
        key_id = upload_response.json()['publicKeyId']
        assert key_id is not None

        ## Set Active
        active_response = key_controller.set_active_key(stream_id=new_stream_id, key_id=key_id)
        assert active_response.status_code == 200
        assert self.api_client.get_stream_controller().get_stream(new_stream_id).json()['activeKeyId'] == key_id

    def test_generate_key(self):
        display_name = 'test_generate_key'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']
        key_controller = self.api_client.get_key_controller()
        key_request = {'publicKeyAlgorithm': KeyAlgorithm.CURVE25519.value,
                       'publicKeyEncoding': KeyEncoding.BASE64.value,
                       'publicKeyDescription': "test key integraiton"}
        new_key_resp = key_controller.create_key_server(new_stream_id, key_request)
        pprint.pprint(new_key_resp.json())
        priv = new_key_resp.json()['privateKeyValue']
        pub = new_key_resp.json()['publicKey']['publicKeyValue']

        sb = SealedBox(KeyController.get_public_key(pub))
        unencrypted_data = b'super secret bytes'
        encrypted_data = sb.encrypt(unencrypted_data)
        unseal_box = SealedBox(KeyController.get_private_key(priv))
        assert unseal_box.decrypt(encrypted_data) == unencrypted_data



class ReadRecordRequest:
    """
    Class for creating read record request objects.
    Creating the request with a class helps ensure that the request body
    conforms to the requirements on the API side.
    """

    def __init__(self):
        """
         Constructor
         """
        self.readRecordRequestType = None
        self.recordCount = None
        self.recordId = None
        self.markAsRead = None
        self.recordTimeStampISO = None

    @property
    def to_json(self):
        """
         returns the class properties as a Dict
         :return: (Dict) class properties
         """
        return self.__dict__



class HealthController:
    """
    Manages interactions with the /health endpoints
    """
    def __init__(self, client):
        self.client = client

    def check_health(self):
        """
        GET request ot the /health endpoint
        :return: Requests Response Object
        """
        return self.client.call_api('GET', "/health")



from nacl.encoding import Base64Encoder
from nacl.public import PrivateKey, PublicKey

from streamduo.models.key_pair import KeyPair, KeyEncoding, KeyAlgorithm


class KeyController:
    """
    Provides methods for interacting with the `/key` endpoints
    """

    def __init__(self, client):
        self.client = client

    @staticmethod
    def create_key_local():
        ## create a secret Key
        sk = PrivateKey.generate()
        pk = KeyPair(publicKey=sk.public_key.encode(encoder=Base64Encoder).decode('utf8'),
                     publicKeyAlgorithm=KeyAlgorithm.CURVE25519.value,
                     publicKeyEncoding=KeyEncoding.BASE64.value)
        return  pk, sk.encode(encoder=Base64Encoder).decode('utf8')

    @staticmethod
    def get_public_key(key_string: str) -> PublicKey:
        return PublicKey(public_key=key_string.encode('utf-8'), encoder=Base64Encoder)

    @staticmethod
    def get_private_key(key_string: str) -> PrivateKey:
        return PrivateKey(private_key=key_string.encode('utf-8'), encoder=Base64Encoder)

    def upload_key(self, stream_id, key):
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/keys/",
                                    body=key.to_json())

    def set_active_key(self, stream_id, key_id):
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/keys/active",
                                    body={'keyId': key_id})

    def create_key_server(self, stream_id, key_request):
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/keys/generate",
                                    body=key_request)


import os
import json
from unittest import TestCase

import jsonschema

from streamduo.client import Client
from streamduo.validators.json_schema import JsonValidator
from streamduo.models.schema import SchemaType

class TestStream(TestCase):
    def setUp(self) -> None:
        self.api_client = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET'))

    def test_create_schema(self):

        ##create a stream
        display_name = 'test_stream'
        stream_controller = self.api_client.get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']
        assert stream_request_result.json()['displayName'] == display_name

        ## Add schema to stream
        schema_controller = self.api_client.get_schema_controller()
        with open("car_schema.json", 'r') as file:
            car_schema = json.load(file)
        create_schema_response1 = schema_controller.create_schema(stream_id=stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        new_schema_id = create_schema_response1.json()['schemaId']
        assert new_schema_id is not None

        ## BAD Schema errors out
        car_schema['properties']['Make']['$id'] = "#root/Make 2"
        create_schema_response2 = schema_controller.create_schema(stream_id=stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        assert create_schema_response2.status_code == 400

        ## cleanup schema
        delete_response = schema_controller.delete_schema(schema_id=new_schema_id, stream_id=stream_id)
        assert delete_response.status_code == 204

        ## Cleanup stream
        delete_response_stream = stream_controller.delete_stream(stream_request_result.json()['streamId'])
        assert delete_response_stream.status_code == 204

    def test_read_schema(self):

        ##create a stream
        display_name = 'test_stream'
        stream_controller = self.api_client.get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']
        assert stream_request_result.json()['displayName'] == display_name

        ## Add schema to stream
        schema_controller = self.api_client.get_schema_controller()
        with open("car_schema.json", 'r') as file:
            car_schema = json.load(file)
        create_schema_response1 = schema_controller.create_schema(stream_id=stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        new_schema_id = create_schema_response1.json()['schemaId']
        assert new_schema_id is not None

        ## get schema
        schema_response = schema_controller.get_schema(stream_id=stream_id, schema_id=new_schema_id)
        schema = schema_response.get_schema()
        assert schema['$id'] == 'https://example.com/object1644429127.json'

        ## test an object w schema
        val = JsonValidator()
        val.set_schema(schema=schema)
        valid_rec = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": 31600,
            "Colour": "White",
            "Doors": "4"
        }
        assert val.validate_record(valid_rec)

        bad_rec = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": 31600,
            "Colour": "White"
        }
        with self.assertRaises(jsonschema.exceptions.ValidationError):
            val.validate_record(bad_rec)

        ## cleanup schema
        delete_response = schema_controller.delete_schema(schema_id=new_schema_id, stream_id=stream_id)
        assert delete_response.status_code == 204

        ## Cleanup stream
        delete_response_stream = stream_controller.delete_stream(stream_request_result.json()['streamId'])
        assert delete_response_stream.status_code == 204

    def test_set_active_schema(self):
        ##create a stream
        display_name = 'test_set_active_schema'
        stream_controller = self.api_client.get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']
        assert stream_request_result.json()['displayName'] == display_name

        ## Add schema to stream
        schema_controller = self.api_client.get_schema_controller()
        with open("car_schema.json", 'r') as file:
            car_schema = json.load(file)
        create_schema_response1 = schema_controller.create_schema(stream_id=stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        new_schema_id = create_schema_response1.json()['schemaId']
        assert new_schema_id is not None

        ## Set new Schema active
        set_active_response = schema_controller.set_active_schema(stream_id=stream_id, schema_id=new_schema_id)
        assert set_active_response.json()['status'] == "Success"

        updated_stream = stream_controller.get_stream(stream_id=stream_id).json()
        assert updated_stream['activeSchemaId'] == new_schema_id

    def test_set_active_schema_w_record(self):
        ##create a stream
        display_name = 'test_set_active_schema'
        stream_controller = self.api_client.get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        stream_id = stream_request_result.json()['streamId']
        assert stream_request_result.json()['displayName'] == display_name

        ## Add schema to stream
        schema_controller = self.api_client.get_schema_controller()
        with open("car_schema.json", 'r') as file:
            car_schema = json.load(file)
        create_schema_response1 = schema_controller.create_schema(stream_id=stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        new_schema_id = create_schema_response1.json()['schemaId']
        assert new_schema_id is not None

        ## Set new Schema active
        set_active_response = schema_controller.set_active_schema(stream_id=stream_id, schema_id=new_schema_id)
        assert set_active_response.json()['status'] == "Success"

        updated_stream = stream_controller.get_stream(stream_id=stream_id).json()
        assert updated_stream['activeSchemaId'] == new_schema_id

        ## Send good record
        record_controller = self.api_client.get_record_controller()
        good_record = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": 31600,
            "Colour": "White",
            "Doors": "4"
        }
        good_write_result = record_controller.write_record(stream_id=stream_id, json_payload=good_record)
        assert good_write_result.status_code == 200

        bad_record = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": 31600,
            "Colour": "White",
            "Doors": 4
        }
        bad_write_result = record_controller.write_record(stream_id=stream_id, json_payload=bad_record)
        assert bad_write_result.status_code == 422




import base64
import hashlib
import json
import pprint
from unittest import TestCase
from streamduo.api.batch import BatchController

import os

class TestJson(TestCase):

    def test_construct_init(self):
        FILE_SIZE = 1024 * 1024 * 100  # 100 mb
        big_file = "unit_test_data/bigfile.dat"
        # multiply file
        with open(big_file, 'wb') as out_file:
            out_file.write(os.urandom(FILE_SIZE))

        req = BatchController.construct_batch_init_request(big_file)
        pprint.pprint(req.to_json())
        assert len(req.hashes.keys()) == 20
        # cleanup
        os.remove(big_file)

    def test_get_part_binary(self):
        FILE_SIZE = 1024 * 1024 * 100  # 100 mb
        big_file = "unit_test_data/bigfile.dat"
        PART_NUMBER = '4'
        # multiply file
        with open(big_file, 'wb') as out_file:
            out_file.write(os.urandom(FILE_SIZE))
        bin_part = BatchController.get_part_binary(file_path=big_file, part_number=PART_NUMBER)

        req = BatchController.construct_batch_init_request(big_file)

        assert req.hashes[int(PART_NUMBER)] == base64.b64encode(hashlib.md5(bin_part).digest()).decode()






from streamduo.validators.json_schema import validate_schema
from streamduo.models.schema import Schema


class SchemaController:
    """
    Class for managing interaction with the /schema endpoints
    """
    def __init__(self, client):
        """
        Constructor, must be instantiated with a client to provide auth support
        :param client: (Client) client object
        """
        self.client = client

    def create_schema(self, stream_id, schema, schema_type):
        """
        Creates a new Schema
        :param schema: (DICT) Desired schema
        :param stream_id: (STRING) stream to attach schema to
        :param schema_type (STRING) type of schema (JSON, AVRO, GREAT_EXPECTATIONS)
        :return: (Requests Response) Response from API call, Body of response is a Schema Object
        """
        validate_schema(schema)
        request_body = {'schema': schema,
                        'schemaType': schema_type.value}
        return self.client.call_api('POST',
                                    f"/streams/{stream_id}/schemas",
                                    body=request_body)

    def delete_schema(self, stream_id, schema_id):
        """
        Deletes a Schema
        :param schema_id: (STRING) Desired schema to delete
        :param stream_id: (STRING) Desired stream of schema to delete
        :return: (Requests Response) Response from API call, Body of response is null.
        """
        return self.client.call_api('DELETE',
                                    f"/streams/{stream_id}/schemas/{schema_id}/")

    def get_schema(self, stream_id, schema_id):
        """
        Gets a Schema
        :param schema_id: (STRING) Desired schema to get
        :param stream_id: (STRING) Desired stream of schema to delete
        :return: (Requests Response) Response from API call.
        """
        resp = self.client.call_api('GET',
                                    f"/streams/{stream_id}/schemas/{schema_id}/")
        schema = Schema(**resp.json())
        return schema

    def set_active_schema(self, stream_id, schema_id):
        """
        Sets the active schema on a stream
        :param stream_id: Desired stream to set Schema to
        :param schema_id: Desired schema ID to set active
        :return: (Requests Response) Response from API call.
        """
        request_body = {"schemaId": schema_id}
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/schemas/active",
                                    body=request_body)




from unittest import TestCase

import fastavro

from streamduo.validators.avro_schema import AvroValidator


class TestJson(TestCase):

    def test_json_record_validation(self):
        val = AvroValidator()
        val.set_local_schema("../test_schemas/car_schema.avsc")
        valid_rec = {
            "Make": "Nissan",
            "Price": "$9,700.00",
            "Odometer (KM)": "31600",
            "Colour": "White",
            "Doors": "4"
        }
        assert val.validate_record(valid_rec)
        inval_rec = valid_rec.copy()
        inval_rec['Odometer (KM)'] = "xyz"
        with self.assertRaises(fastavro._validate_common.ValidationError):
            val.validate_record(inval_rec)


class StreamController:
    """
    Class for managing interaction with the /stream endpoints
    """
    def __init__(self, client):
        """
        Constructor, must be instantiated with a client to provide auth support
        :param client: (Client) client object
        """
        self.client = client

    def create_stream(self, stream_display_name):
        """
        Creates a new Stream
        :param stream_display_name: (String) Desired Display name of new stream
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        request_body = {'displayName': stream_display_name}
        return self.client.call_api('POST',
                                    "/stream",
                                    body=request_body)

    def get_stream(self, stream_id):
        """
        Gets the details of a stream
        :param stream_id: (String) Stream ID
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        return self.client.call_api('GET',
                                    f"/stream/{stream_id}")

    def delete_stream(self, stream_id):
        """
        Deletes a Stream
        :param stream_id: (String) Stream ID
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        return self.client.call_api('DELETE',
                                    f"/stream/{stream_id}")

    def add_new_machine_client_to_stream(self, stream_id, client_display_name,
                                         is_producer=False, is_consumer=False):
        """
        Creates a new Client ID, and grants that client permissions on a Stream
        :param stream_id: (String) Stream ID to add the client to
        :param client_display_name: (String) Desired display name of the client
        :param is_producer:  (Boolean) Should producer access be granted to this Client
        :param is_consumer: (Boolean) Should consumer access be granted to this Client
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        body = {'actorDisplayName': client_display_name,
                'isProducer': is_producer,
                'isConsumer': is_consumer,
                'actorType': 'CLIENT'
                }
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def add_public_client_to_stream(self, stream_id, client_display_name,
                                         is_producer=False, is_consumer=False):
        """
        Creates a new Client ID, and grants that client permissions on a Stream
        :param stream_id: (String) Stream ID to add the client to
        :param client_display_name: (String) Desired display name of the client
        :param is_producer:  (Boolean) Should producer access be granted to this Client
        :param is_consumer: (Boolean) Should consumer access be granted to this Client
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        body = {'actorDisplayName': client_display_name,
                'isProducer': is_producer,
                'isConsumer': is_consumer,
                'actorType': 'PUBLIC'
                }
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def add_user_to_stream(self, stream_id, user_email, is_producer=False, is_consumer=False):
        """
        Adds a human user to a stream with permissions
        :param stream_id:  (String) Stream ID to add the User to
        :param user_email: (String) The Email address of the User to Add
        :param is_producer: (Boolean) Should producer access be granted to this User
        :param is_consumer: (Boolean) Should consumer access be granted to this User
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        body = {'actorDisplayName': user_email,
                'isProducer': is_producer,
                'isConsumer': is_consumer,
                'actorType': 'USER'
                }
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def remove_machine_client_from_stream(self, stream_id, client_id):
        """
        Removes access from a client on a given stream
        :param stream_id: (String) Stream ID to remove client from
        :param client_id: (String) Client ID to remove from Stream
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        body = {'actorId': client_id,
                'actorType': 'CLIENT'
                }
        return self.client.call_api('DELETE',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def remove_user_from_stream(self, stream_id, user_id):
        """
        Removes access from a client on a given stream
        :param stream_id: (String) Stream ID to remove client from
        :param user_id: (String) User ID to remove from Stream
        :return: (Requests Response) Response from API call, Body of response is a Stream Object
        """
        body = {'actorId': user_id,
                'actorType': 'USER'
                }
        return self.client.call_api('DELETE',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)




from setuptools import setup, find_packages

NAME = "streamduo"
VERSION = "0.0.29"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["requests", "jsonschema", "pynacl", "pandas", "great_expectations"]

setup(
    name=NAME,
    version=VERSION,
    description="streamduo.com API",
    author_email="steve@streamduo.com",
    url="",
    keywords=["streamduo.com API"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    SDK for streamduo.com
    """
)



import json
import csv
from fastavro.validation import validate
from fastavro import parse_schema


def validate_schema(schema):
    parsed_schema = parse_schema(schema)


class AvroValidator:
    schema = None
    int_fields = []
    float_fields = []
    """
    Provides methods for validating records and files against a given AVRO schema
    """

    def __init__(self):
        pass

    def set_local_schema(self, schema_file):
        try:
            with open(schema_file, 'r') as file:
                self.schema = parse_schema(json.load(file))
                print(self.schema)
                # integer :: int
                # number :: float
                for v in self.schema['fields']:
                    if 'long' in v['type'] or 'int' in v['type']:
                        self.int_fields.append(v['name'])
                    elif 'float' in v['type'] or 'double' in v['type']:
                        self.float_fields.append(v['name'])
        except SyntaxError as error:
            raise

    def set_schema(self, schema):
        self.schema = parse_schema(schema)
        print(self.schema)

    def validate_record(self, record):
        ## retype numerics
        try:
            for x in self.int_fields:
                record[x] = int(record[x])
            for y in self.float_fields:
                record[y] = float(record[y])
        except:
            pass
        return validate(record, self.schema)



import os

import requests

from streamduo.api.batch import BatchController
from streamduo.api.health import HealthController
from streamduo.api.key import KeyController
from streamduo.api.stream import StreamController
from streamduo.api.actor import ActorController
from streamduo.api.record import RecordController
from streamduo.api.schema import SchemaController


class Client:
    """
    The Client Object manages authorization headers for API calls.
    controllers are generated from this Client Object, and are provided
    with the authorization support for their methods.
    """
    auth_endpoint = "https://streamduo-authentication.auth.us-east-1.amazoncognito.com/oauth2/token"
    api_endpoint = "https://api.streamduo.com"
    stream_scope = "https://api.streamduo.com/manage:streams"
    record_scope = "https://api.streamduo.com/read:records"

    def __init__(self, client_id, client_secret):
        """
        Constructor for Client object.
        :param client_id: StreamDuo Client ID
        :param client_secret: StreamDuo Client Secret
        """
        self.auth_endpoint = Client.auth_endpoint
        self.scope = ''
        if os.getenv('STREAMDUO_SDK_URL'):
            self.api_endpoint = os.getenv('STREAMDUO_SDK_URL')
        else:
            self.api_endpoint = Client.api_endpoint


        self.client_id = client_id
        self.client_secret = client_secret
        self.token = None
        self.auth_req_header = {'content-type': 'application/x-www-form-urlencoded'}
        self.token_req_payload = {'grant_type': 'client_credentials',
                                  'client_id': self.client_id,
                                  'client_secret': self.client_secret}

    def set_oauth_token(self, scope):
        """
        Method to set the OAUTH token
        :return: None
        """
        self.scope = scope
        self.token_req_payload['scope'] = self.scope
        try:
            token_response = requests.post(self.auth_endpoint,
                                           data=self.token_req_payload,
                                           headers=self.auth_req_header)
            self.token = token_response.json()['access_token']

        except KeyError:
            self.token = None

    def call_api(self, verb, path, params=None, body=None, files=None):
        """
        Generic function to call the SteamDuo APIs.
        Authorization and headers are managed prior to calling API.
        :param verb: HTTP Verb (GET, POST, DELETE)
        :param path: URL path excluding base URL (i.e. /stream)
        :param params: dict of querystring params
        :param body: Body of request if one is used. This can be either a dict, or a JSON formatted String.
        :param files: A File Object of any file used in the API call.
        :return: Requests Response Object of the API call.
        """
        header = {'authorization': f"Bearer {self.token}",
                  'content-type': 'application/json'}
        if verb == 'GET':
            return requests.get(f"{self.api_endpoint}{path}",
                                params=params,
                                headers=header)
        if verb == 'POST':
            if files:
                del header['content-type']
            return requests.post(f"{self.api_endpoint}{path}",
                                 headers=header,
                                 files=files,
                                 json=body)
        if verb == 'DELETE':
            return requests.delete(f"{self.api_endpoint}{path}",
                                   headers=header,
                                   json=body)
        return None

    def get_health_controller(self):
        """
        Provides a health controller to access /health endpoints
        :return: HealthController
        """
        if self.scope != Client.stream_scope or self.token is None:
            self.set_oauth_token(Client.stream_scope)
        return HealthController(self)

    def get_stream_controller(self):
        """
        Provides a Stream controller to access /stream endpoints
        :return: StreamController
        :return:
        """
        if self.scope != Client.stream_scope or self.token is None:
            self.set_oauth_token(Client.stream_scope)
        return StreamController(self)

    def get_actor_controller(self):
        """
        Provides an Actor controller to access /client endpoints.
        :return: ActorController
        """
        if self.scope != Client.stream_scope or self.token is None:
            self.set_oauth_token(Client.stream_scope)
        return ActorController(self)

    def get_record_controller(self):
        """
        Provides a Record Controller to interact with reading/writing streams
        :return: RecordController
        """
        if self.scope != Client.record_scope or self.token is None:
            self.set_oauth_token(Client.record_scope)
        return RecordController(self)

    def get_schema_controller(self):
        """
        Provides a Schema Controller to interact with schema endpoints
        :return: SchemaController
        """
        if self.scope != Client.stream_scope or self.token is None:
            self.set_oauth_token(Client.record_scope)
        return SchemaController(self)

    def get_batch_controller(self):
        if self.scope != Client.record_scope or self.token is None:
            self.set_oauth_token(Client.record_scope)
        return BatchController(self)

    def get_key_controller(self):
        """
        Provides a Record Controller to interact with reading/writing streams
        :return: RecordController
        """
        return KeyController(self)


class PublicClient:
    """
    The Client Object manages authorization headers for API calls.
    controllers are generated from this Client Object, and are provided
    with the authorization support for their methods.
    """
    auth_endpoint = "https://streamduo-auth.auth.us-east-1.amazoncognito.com/oauth2/token"
    api_endpoint = "https://api.streamduo.com"

    def __init__(self):
        """
        Constructor for Public Client object.
        """
        if os.getenv('STREAMDUO_SDK_URL'):
            self.api_endpoint = os.getenv('STREAMDUO_SDK_URL')
        else:
            self.api_endpoint = Client.api_endpoint

    def call_api(self, verb, path, body=None, files=None):
        """
        Generic function to call the SteamDuo APIs.
        Authorization and headers are managed prior to calling API.
        :param verb: HTTP Verb (GET, POST, DELETE)
        :param path: URL path excluding base URL (i.e. /stream)
        :param body: Body of request if one is used. This can be either a dict, or a JSON formatted String.
        :param files: A File Object of any file used in the API call.
        :return: Requests Response Object of the API call.
        """
        header = {'content-type': 'application/json'}
        if verb == 'POST':
            if files:
                del header['content-type']
            return requests.post(f"{self.api_endpoint}/public{path}",
                                 headers=header,
                                 files=files,
                                 json=body)
        return None

    def get_record_controller(self):
        """
        Provides a Record Controller to interact with reading/writing streams
        :return: RecordController
        """
        return RecordController(self)






import json
import os
import pprint
from unittest import TestCase

from streamduo.client import Client
from streamduo.models.batch_data import BatchData
from streamduo.api.batch import BatchController
from streamduo.models.schema import SchemaType


class TestBatch(TestCase):

    def setUp(self) -> None:
        self.api_client = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET'))

    def test_init_batch(self):
        display_name = 'batch_test_stream'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']

        batch_controller = self.api_client.get_batch_controller()
        FILE_SIZE = 1024 * 1024 * 100  # 1GB
        big_file = "bigfile.dat"
        # multiply file
        with open(big_file, 'wb') as out_file:
            out_file.write(os.urandom(FILE_SIZE))

        resp = batch_controller.send_batch_init(stream_id=new_stream_id, file_path=big_file)
        batch_data = BatchData(**resp.json())
        assert len(batch_data.outstandingParts.keys()) == 20
        assert batch_data.streamSchema == None
        assert batch_data.requires_validation() == False

        ## Add schema to stream
        schema_controller = self.api_client.get_schema_controller()
        with open("car_schema.json", 'r') as file:
            car_schema = json.load(file)
        create_schema_response1 = schema_controller.create_schema(stream_id=new_stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        new_schema_id = create_schema_response1.json()['schemaId']

        ## Set Active Schema
        set_active_response = schema_controller.set_active_schema(stream_id=new_stream_id, schema_id=new_schema_id)

        ## Create new Req
        resp = batch_controller.send_batch_init(stream_id=new_stream_id, file_path=big_file)
        batch_data = BatchData(**resp.json())
        assert len(batch_data.outstandingParts.keys()) == 20
        assert batch_data.requires_validation() == True
        assert batch_data.get_stream_schema().get_schema()['$id'] == 'https://example.com/object1644429127.json'

        ## Cleanup
        stream_controller.delete_stream(new_stream_id)

    def test_send_part(self):
        display_name = 'batch_test_stream'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']

        batch_controller = self.api_client.get_batch_controller()
        FILE_SIZE = 1024 * 1024 * 100  # 1GB
        big_file = "bigfile.dat"
        # multiply file
        with open(big_file, 'wb') as out_file:
            out_file.write(os.urandom(FILE_SIZE))

        resp = batch_controller.send_batch_init(stream_id=new_stream_id, file_path=big_file)
        batch_data = BatchData(**resp.json())
        part_bin = BatchController.get_part_binary(file_path=big_file, part_number=5)
        ## Send part 5
        batch_data_resp = batch_controller.send_batch_part(batch_data=batch_data, part_number=5,
                                                           binary_payload=part_bin)
        batch_data_2 = BatchData(**batch_data_resp.json())
        pprint.pprint(batch_data_2.to_json())
        assert '5' not in batch_data_2.outstandingParts.keys()

        ## Cleanup
        stream_controller.delete_stream(new_stream_id)

    def test_full_file(self):
        display_name = 'batch_test_stream_full'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']

        batch_controller = self.api_client.get_batch_controller()
        FILE_SIZE = 1024 * 1024 * 20  # 20MB
        big_file = "bigfile.dat"
        # multiply file
        with open(big_file, 'wb') as out_file:
            out_file.write(os.urandom(FILE_SIZE))

        batch_data = batch_controller.send_file(stream_id=new_stream_id, file_path=big_file)
        assert len(batch_data.outstandingParts.keys()) == 0
        ## Cleanup
        stream_controller.delete_stream(new_stream_id)

    def test_full_file_schema(self):
        display_name = 'batch_test_stream_full_schema'
        stream_controller = self.api_client.get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']

        ## Add schema to stream
        schema_controller = self.api_client.get_schema_controller()
        with open("car_schema.json", 'r') as file:
            car_schema = json.load(file)
        create_schema_response1 = schema_controller.create_schema(stream_id=new_stream_id,
                                                                  schema=car_schema, schema_type=SchemaType.JSON)
        new_schema_id = create_schema_response1.json()['schemaId']

        ## Set Active Schema
        schema_controller.set_active_schema(stream_id=new_stream_id, schema_id=new_schema_id)

        ## Send File
        batch_controller = self.api_client.get_batch_controller()
        batch_data = batch_controller.send_file(stream_id=new_stream_id, file_path="../test_data/car_sales.csv")
        assert len(batch_data.outstandingParts.keys()) == 0

        ## Cleanup
        stream_controller.delete_stream(new_stream_id)



class BatchInitRequest:
    """
    Class for creating batch file inti request objects.
    Creating the request with a class helps ensure that the request body
    conforms to the requirements on the API side.
    """

    def __init__(self):
        """
         Constructor
         """
        self.fileName = None
        self.hashes = {}
        self.hashValue = None
        self.fileType = None
        self.serverSideValidation = False
        self.totalParts = None

    def to_json(self):
        return self.__dict__